import os, sys
import random
# import setuptools
import shutil
import cv2
from os.path import join, exists, dirname
from pprint import pprint as pp
# print(os.getcwd()) # Thư mục console đang chạy
# print(__file__) # Thư mục file code
codeDir=dirname(os.path.abspath(__file__))
sys.path.append(codeDir)

# pp(sys.path)

from ParamsBase import tactParametters
from tqdm import tqdm
from yolo_boxes import yolo_str_to_xyxy

APP_NAME='Image_Augmentation'

class Parameters(tactParametters):
    def __init__(self, ModuleName="TACT"):
        super().__init__(saveParam_onlyThis_APP_NAME=True)
        self.AppName = APP_NAME
        self.Ready_to_run = False # Nếu bắt buộc phải config thì đặt cái này = False, khi nào user chỉnh sang True thì mới cho chạy
        self.HD = {
            "Brightness": "Adjust brightness (value: -255 to 255)",
            "Shadow_Brightness": "Adjust Shadow_Brightness (value: -255 to 255)",
            "Contrast": "Adjust contrast (alpha > 1 increases contrast, 0 < alpha < 1 decreases)",
            "Saturation": "Convert to HSV, adjust saturation, then back to BGR",
            "Hue": "Convert to HSV, adjust hue, then back to BGR",
            "cover_yolo_string": "'0 0.496094 0.772135 0.527344 0.453125' (ví dụ xâu), cái này nếu có giá trị, nó sẽ crop ảnh theo tọa độ trong này, TODO: tính lại tọa độ label theo cái này",
            "cover_yolo_string__resize_after_crop_HW": "Là 1 list, nếu không muốn resize, để thành list rỗng: []",
            "Max_random_image_to_Aug=x": "x==0: Aug tất cả các ảnh, x>0: chỉ Aug random x ảnh thôi, để test trong quá trình dò tim các tham số",
            "Cách dùng 1": "lệnh chạytrong CMD: 'ntanh_aug', khi hiển thị ảnh: bấm SPACE để tạm dừng/chạy tiếp, bấm ESC để thoát",
            "Cách dùng 2": "Muốn đưa tham số nào về mặc định thì xóa nó đi rồi chạy lại",
        }
        self.Intro="Chương trình này dành riêng cho việc augmentation ảnh nhằm tạo ra ảnh có các kiểu khác nhau phục vụ cho mục đích training model"

        self.cover_yolo_string = ""
        self.cover_yolo_string__expand_n_pixel = 20
        self.cover_yolo_string__resize_after_crop_WH=[]
        self.Brightness = 50
        self.Shadow_Brightness=0
        self.Contrast = 1.2
        self.Saturation = 0
        self.Hue = 0
        self.image_folder__input = ""
        self.image_folder_output = ""
        self.Copy_label_when_save_augment_image=True

        self.Display_image=True
        self.Display_image_Stop_to_View_in___cv2_waitkey__ms=1000
        self.Display_image_max_height=800
        self.Max_random_image_to_Aug=10
        self.load_then_save_to_yaml(file_path=f"{APP_NAME}.yml", ModuleName=ModuleName)


mParams = Parameters("Augmentation_images")

def taImshow(title="image",image=None, wait=0):
    H,W=image.shape[:2]
    if H > mParams.Display_image_max_height:
        fxy = mParams.Display_image_max_height / H
        image = cv2.resize(image, None, fx=fxy, fy=fxy)
    cv2.imshow(title, image)
    cv2.waitKey(wait)

from PIL import Image


def update_labels(
    label_path, x_min_crop, y_min_crop, new_imW, new_imH, old_imW, old_imH
):
    # Cập nhật file nhãn dựa trên ảnh đã crop
    new_labels = []
    with open(label_path, "r") as file:
        lines = file.readlines()
        for line in lines:
            # print(line)
            obj_class, x_center, y_center, width, height = map(float, line.split())

            # Tính tọa độ gốc của bounding box (theo ảnh ban đầu)
            old_x_min = (x_center - width / 2) * old_imW
            old_y_min = (y_center - height / 2) * old_imH
            old_x_max = (x_center + width / 2) * old_imW
            old_y_max = (y_center + height / 2) * old_imH

            # Điều chỉnh bounding box theo vùng đã crop
            new_x_min = max(0, old_x_min - x_min_crop)
            new_y_min = max(0, old_y_min - y_min_crop)
            new_x_max = min(new_imW, old_x_max - x_min_crop)
            new_y_max = min(new_imH, old_y_max - y_min_crop)

            # Bỏ qua các đối tượng không nằm trong vùng crop
            # if new_x_min < new_x_max and new_y_min < new_y_max:
            if 1:
                new_x_center = (new_x_min + new_x_max) / 2 / new_imW
                new_y_center = (new_y_min + new_y_max) / 2 / new_imH
                new_width = (new_x_max - new_x_min) / new_imW
                new_height = (new_y_max - new_y_min) / new_imH

                # Kiểm tra nếu tọa độ trung tâm nằm trong vùng hợp lệ
                # if 0 <= new_x_center <= 1 and 0 <= new_y_center <= 1:
                new_labels.append(
                    f"{int(obj_class)} {new_x_center:0.6f} {new_y_center:0.6f} {new_width:0.6f} {new_height:0.6f}"
                )

    s = "\n".join(new_labels)
    return s


class ImageAugmentation:
    def __init__(self):
        pass

    def Change_Brightness(self, image, value):
        # Adjust brightness
        hsv = cv2.cvtColor(image, cv2.COLOR_BGR2HSV)
        h, s, v = cv2.split(hsv)

        # Ensure brightness value is in the range of [-255, 255]
        if value > 0:
            lim = 255 - value
            v[v > lim] = 255
            v[v <= lim] += value
        else:
            lim = -value
            v[v < lim] = 0
            v[v >= lim] += value

        final_hsv = cv2.merge((h, s, v))
        result = cv2.cvtColor(final_hsv, cv2.COLOR_HSV2BGR) 
        return result

    def Change_Shadow_Brightness(self, image, value):
        hsv = cv2.cvtColor(image, cv2.COLOR_BGR2HSV)
        h, s, v = cv2.split(hsv)

        # Create a mask for shadows (dark areas)
        shadow_mask = v < 128

        # Adjust brightness in shadow areas
        if value > 0:
            lim = 255 - value
            v[shadow_mask & (v > lim)] = 255
            v[shadow_mask & (v <= lim)] += value
        else:
            lim = -value
            v[shadow_mask & (v < lim)] = 0
            v[shadow_mask & (v >= lim)] += value

        final_hsv = cv2.merge((h, s, v))
        result = cv2.cvtColor(final_hsv, cv2.COLOR_HSV2BGR)

        return result

    def Change_Contrast(self, image, value):
        # Adjust contrast (alpha > 1 increases contrast, 0 < alpha < 1 decreases)
        return cv2.convertScaleAbs(image, alpha=value, beta=0)

    def Change_Saturation(self, image, value):
        # Convert to HSV, adjust saturation, then back to BGR
        hsv = cv2.cvtColor(image, cv2.COLOR_BGR2HSV)
        hsv[:, :, 1] = cv2.add(hsv[:, :, 1], value)
        return cv2.cvtColor(hsv, cv2.COLOR_HSV2BGR)

    def Change_Hue(self, image, value):
        # Convert to HSV, adjust hue, then back to BGR
        hsv = cv2.cvtColor(image, cv2.COLOR_BGR2HSV)
        hsv[:, :, 0] = cv2.add(hsv[:, :, 0], value)
        return cv2.cvtColor(hsv, cv2.COLOR_HSV2BGR)

    def Change_Vibrance(self, image, value):
        # Custom vibrance logic, placeholder here
        return image  # Vibrance adjustment not implemented for simplicity

    def Change_image_multiple_ways(self, image,image_path, options):
        newLabel=''
        if options.get("cover_yolo_string", False):            
            imH, imW = image.shape[:2]
            id, x1, y1, x2, y2 = yolo_str_to_xyxy(
                yolo_str=mParams.cover_yolo_string, imH=imH, imW=imW
            )

            d = mParams.cover_yolo_string__expand_n_pixel
            y1, y2,  x1, x2= y1-d,y2+d, x1-d,x2+d

            image = image[y1:y2, x1:x2]

            # TODO: tính lại tọa độ theo cái này:
            label_path = image_path.replace(".jpg", ".txt")
            if exists(label_path): 
                x_min_crop, y_min_crop= x1, y1 
                new_imH, new_imW = image.shape[:2]
                newLabel = update_labels(
                    label_path,
                    x_min_crop,
                    y_min_crop,
                    new_imW,
                    new_imH,
                    imW,
                    imH,
                )
            if mParams.cover_yolo_string__resize_after_crop_WH:
                image = cv2.resize(
                    image, mParams.cover_yolo_string__resize_after_crop_WH
                )

        if mParams.Display_image: taImshow(title="Org image", image=image, wait=1)
        if options.get("Brightness", False):            
            image = self.Change_Brightness(image, mParams.Brightness)
            if mParams.Display_image: taImshow(title="Brightness", image=image, wait=1)
        if options.get("Shadow_Brightness", False):   
            image = self.Change_Shadow_Brightness(image, mParams.Shadow_Brightness)
            if mParams.Display_image: taImshow(title="image3", image=image, wait=0)
        if options.get("Contrast", False):
            image = self.Change_Contrast(image, mParams.Contrast)
            if mParams.Display_image: taImshow(title="Contrast", image=image, wait=1)
        if options.get("Saturation", False):            
            image = self.Change_Saturation(image, mParams.Saturation)
            if mParams.Display_image: taImshow(title="Saturation", image=image, wait=1)
        if options.get("Hue", False):            
            image = self.Change_Hue(image, mParams.Hue)
            if mParams.Display_image: taImshow(title="Hue", image=image, wait=1)

        if mParams.Display_image:
            keyPressed=cv2.waitKey(mParams.Display_image_Stop_to_View_in___cv2_waitkey__ms)
            if keyPressed==32:
                while True:
                    keyPressed = cv2.waitKey(
                        mParams.Display_image_Stop_to_View_in___cv2_waitkey__ms
                    )
                    if keyPressed==32:
                        break
                    if keyPressed==27:
                        os._exit(1)

        return image, newLabel

    def Augment_folder(self, **options):
        if mParams.image_folder__input == "":
            print("Hãy cấu hình giá trị cho file tham số trước, rồi chạy lại.")
            os.startfile("configs_ntanh_libs.yml")
            return
        image_folder_input = mParams.image_folder__input
        image_folder_output = mParams.image_folder_output

        FIS = mParams.fnFIS(image_folder_input, exts=(".jpg",))
        if mParams.Max_random_image_to_Aug > 0:
            FIS = [random.choice(FIS) for _ in range(mParams.Max_random_image_to_Aug)]
        nImg = len(FIS)

        if not exists(image_folder_output):
            os.makedirs(image_folder_output, exist_ok=True)

        for image_path in tqdm(FIS):
            image = cv2.imread(image_path)
            if image is None:
                print(f"Failed to read image: {image_path}")
                continue

            image_out, newLabel = self.Change_image_multiple_ways( image, image_path, options )
            fnOut = image_path.replace(image_folder_input, image_folder_output)
            label_inp = image_path.replace(".jpg", ".txt")
            label_out = fnOut.replace('.jpg', '.txt')

            os.makedirs(dirname(fnOut), exist_ok=True) 
            cv2.imwrite(fnOut, image_out)
            if mParams.Copy_label_when_save_augment_image:
                if newLabel:
                    with open(label_out, "w") as file:
                        file.write(newLabel)
                elif exists(label_inp):
                    shutil.copy(label_inp, label_out) 
            # print(".", end="", flush=True)

        print()
        print(f"Done augmenting {nImg} images to {image_folder_output}")

def Aug_Folder():
    if not mParams.Ready_to_run:
        print('Thay đổi tham số config trong file:', mParams.get_Home_Dir())
        return
    
    aug = ImageAugmentation()
    print("cover_yolo_string:", mParams.get("cover_yolo_string", False))
    print("Brightness:", mParams.get("Brightness", False))
    print("Contrast:", mParams.get("Contrast", False))
    print("Saturation:", mParams.get("Saturation", False))
    print("Hue:", mParams.get("Hue", False))
    aug.Augment_folder(
        cover_yolo_string=mParams.get("cover_yolo_string", False),
        Brightness=mParams.get("Brightness", False),
        Contrast=mParams.get("Contrast", False),
        Saturation=mParams.get("Saturation", False),
        Hue=mParams.get("Hue", False),
        Vibrance=mParams.get("Vibrance", False),
    )
if __name__ == "__main__":
    Aug_Folder()



# from ruamel import yaml
# from ruamel.yaml import YAML
# # ruamel.yaml.__version__
# # Out[15]: '0.18.6'
#
import pathlib
import sys
from os.path import exists, join
import os
from datetime import datetime
from os.path import dirname
from pprint import pprint

from ruamel.yaml import YAML

yaml = YAML()
# yaml.indent(offset=4)
yaml.compact(seq_seq=False, seq_map=False)

yml_file_contents = {}


def delkeyVal(mDict, lstmkey):
    for mkey in lstmkey:
        if mkey in mDict:
            del mDict[mkey]
    return mDict

from os.path import join, exists, dirname, basename
class tactParametters:
    def __init__(self, logdir="", saveParam_onlyThis_APP_NAME=False):
        self.ModuleName = "TACT"
        self.logdir = logdir
        self.fn = ""
        self.AppName = ""
        self.saveParam_onlyThis_APP_NAME = saveParam_onlyThis_APP_NAME
        # self.debug_config_path = "AI_Data/debug.yml"

    def to_yaml(self, file_path):
        global yml_file_contents
        if self.AppName:
            pHome = get_Home_Dir(self.AppName)
            file_path = os.path.join(pHome, basename(file_path))
        with open(file_path, "w", encoding="utf-8") as file:
            # mdict = {self.ModuleName: self.__dict__}
            mDict = self.__dict__.copy()
            mDict = delkeyVal(
                mDict,
                [
                    "ModuleName",
                    "logdir",
                    "fn",
                    "AppName",
                    "saveParam_onlyThis_APP_NAME",
                ],
            )
            # print("self.saveParam_onlyThis_APP_NAME", self.saveParam_onlyThis_APP_NAME)
            if self.saveParam_onlyThis_APP_NAME:
                yaml.dump({self.ModuleName: mDict}, file)
            else:
                yml_file_contents[self.ModuleName] = mDict
                yaml.dump(yml_file_contents, file)

    def from_yaml(self, file_path):
        global yml_file_contents
        if self.AppName:
            pHome=get_Home_Dir(self.AppName)
            file_path = os.path.join(pHome, basename(file_path))
        if exists(file_path):
            with open(file_path, "r", encoding="utf-8") as file:
                xxx = yaml.load(file)
                if xxx:
                    yml_file_contents = xxx
                if yml_file_contents:
                    if self.ModuleName in yml_file_contents:
                        data = yml_file_contents[self.ModuleName]
                        self.__dict__.update(data)

    def load_then_save_to_yaml( self, file_path, ModuleName="TACT", flogDict=False ):
        self.ModuleName = ModuleName
        self.fn = file_path
        self.from_yaml(file_path)
        self.to_yaml(file_path)
        if flogDict:
            self.ta_print_log(str(self.__dict__))

    def save_to_yaml_only(self, filepath=None):
        if filepath is not None:
            self.fn = filepath
        self.to_yaml(self.fn)
    def get(self, key, default=None):
        if key in self.__dict__:
            return self.__dict__[key]
        else:
            return default
    @staticmethod
    def fnFIS(mDir, exts=(".jpg", ".jpeg", ".png")):  # mConf.local_log_data_dir_input
        inputTypes_Files = []
        for D, _, F in os.walk(mDir):
            for fn in F:
                if fn.endswith(exts):
                    inputTypes_Files.append(join(D, fn).replace("\\", "/"))
        inputTypes_Files.sort()
        return inputTypes_Files

    def ta_print_log(self, *args):
        logdir = self.logdir
        margs = []
        for x in args:
            try:
                margs.append(str(x))
            except:
                pass
        s = " ".join(margs)
        currTime = datetime.now()
        sDT = currTime.strftime("%m/%d, %H:%M:%S")
        if logdir:
            fn = f"{logdir}/logs/{currTime.year}/{currTime.month}/{currTime.day}/logs.txt"
        else:
            fn = f"logs/{currTime.year}/{currTime.month}/{currTime.day}/logs.txt"
        os.makedirs(dirname(fn), exist_ok=True)
        with open(fn, "a", encoding="utf-8") as ff:
            ff.write(f"{s}\n")
        print(sDT, s)

    def get_Home_Dir(self,AppName=None):
        if not AppName:
            AppName = self.AppName
        return get_Home_Dir(AppName)


def get_Home_Dir(AppName):
    drive = pathlib.Path.home().drive 
    folder_path = join(drive, "/RunProgram", AppName).replace("\\", "/")
    os.makedirs(folder_path, exist_ok=True)
    return folder_path

"""

from ntanh.ParamsBase import tactParametters
APP_NAME='TACT_Main'

class Parameters(tactParametters):
    def __init__(self, ModuleName="TACT"):
        super().__init__(saveParam_onlyThis_APP_NAME=False)
        self.AppName = APP_NAME
        # self.Ready_to_run = False # Nếu bắt buộc phải config thì đặt cái này = False, khi nào user chỉnh sang True thì mới cho chạy
        self.HD = {
            "Mô tả": "Chương trình này nhằm xây dựng tham số cho các chương trình khác",            
        }         
        self.load_then_save_to_yaml(file_path=f"{APP_NAME}.yml", ModuleName=ModuleName)


mParams = Parameters("TACT_Module")

"""



"""
if __name__ == "__main__":
    RUN = "Test Normal"
    if RUN == "Test Base":
        mcls = tactParametters()
        mcls.load_then_save_to_yaml("tmp.yml")
    if RUN == "Short":
        # from taFuncs.ParamsBase import tactParametters
        class Config(tactParametters):
            def __init__(self):
                super().__init__()

                self.load_then_save_to_yaml(
                    "system_config.yml", ModuleName="SystemParams"
                )

    if RUN == "Test Normal":

        TACT = 1  #### <<<<<=================== Sửa dòng này =0 để chạy không cần lưu tham số nhá, muốn thêm tham số, chỉ việc thêm vào class bên dưới
        if TACT:
            # from aiLibs.paramettersbase import taParametters, ta_print_log
            pass
        else:

            class taParametters:
                def __init__(self):
                    pass

            def ta_print_log(*args):
                print(*args)

        # ----------------------------------------------------------------------------------------------------------
        class Parameters(tactParametters):
            def __init__(self, ModuleName="TACT"):
                super().__init__()
                self.thamso1 = "thamso1"
                self.thamso2 = " xâu tiếng việt"
                self.api_url = "https://200.168.90.38:6699/avi/collect_data"
                self.testpath = "D:/test_debug_fii"
                self.test_real = 0.8
                self.test_int = 12
                self.test_dict = {
                    1: 2,
                    3: 4.5,
                    "6": "bảy nhá",
                    -1: "Tám",
                    9: [10, 11.2, "22", (33, 44, "55")],
                    10: {101: 12, 102: "mười ba"},
                }
                self.test_list = [1, 2, 3, 4, 5, 6, 7, 8, 9]

                fn = "configs_test.yml"
                self.load_then_save_to_yaml(fn, ModuleName=ModuleName)

        mParams = Parameters("test")

        class Parameters(tactParametters):
            def __init__(self, ModuleName="TACT"):
                super().__init__()
                self.thamso1 = "thamso1"
                self.thamso2 = " xâu tiếng việt"
                self.api_url = "https://200.168.90.38:6699/avi/collect_data"
                self.testpath = "D:/test_debug_fii"
                self.test_real = 0.8
                self.test_int = 12
                self.test_dict = {
                    1: 2,
                    3: 4.5,
                    "6": "bảy nhá",
                    -1: "Tám",
                    9: [10, 11.2, "22", (33, 44, "55")],
                    10: {101: 12, 102: "mười ba"},
                }
                self.test_list = [1, 2, 3, 4, 5, 6, 7, 8, 9]

                fn = "configs_test.yml"
                if TACT:
                    self.load_then_save_to_yaml(fn, ModuleName=ModuleName)

        mParams = Parameters(ModuleName="Test module")
        mParams1 = Parameters(ModuleName="Test module Mot")
        pprint(mParams.__dict__)
        pprint(mParams1.__dict__)
        print(mParams.api_url)
        print(mParams1.api_url)

        pass

    #
    #
    

"""


def xyxy_to_yolo_str(obj_class, x1, y1, x2, y2, imH, imW):
    # Tọa độ trung tâm
    x_center = (x1 + x2) / 2 / imW
    y_center = (y1 + y2) / 2 / imH
    # Chiều rộng và chiều cao
    width = (x2 - x1) / imW
    height = (y2 - y1) / imH

    # Tạo chuỗi YOLO: "class x_center y_center width height"
    yolo_str = f"{obj_class} {x_center:.6f} {y_center:.6f} {width:.6f} {height:.6f}"

    return yolo_str


def yolo_str_to_xyxy(yolo_str, imH, imW):
    """Chuyển đổi yolo string sang box xyxy

    x1,y1,x2,y2 = ntanh.yolo_str_to_xyxy(yolo_str, imH, imW)

    Args:
        yolo_str (string with id): '0 0.195312 0.416667 0.234375 0.416667'
        imH (int/float): Cao ảnh
        imW (int/float): Rộng ảnh

    Returns:
        int: objclass, x_min, y_min, x_max, y_max hoặc id, x1, y1, x2, y2
    """
    # Tách chuỗi YOLO thành các giá trị
    yolo_data = yolo_str.split()
    obj_class = yolo_data[0]  # Lớp đối tượng (class)
    x_center = float(yolo_data[1])
    y_center = float(yolo_data[2])
    width = float(yolo_data[3])
    height = float(yolo_data[4])

    # Chuyển từ YOLO sang tọa độ góc
    x_min = (x_center - width / 2) * imW
    x_max = (x_center + width / 2) * imW
    y_min = (y_center - height / 2) * imH
    y_max = (y_center + height / 2) * imH

    return int(obj_class), int(x_min), int(y_min), int(x_max), int(y_max)


if __name__ == "__main__":
    # Convert bbox to Yolo và ngược lại:
    class img:
        shape = (600, 800, 3)

    x1 = 50
    y1 = 100
    x2 = 200
    y2 = 300
    imH, imW = img.shape[:2]
    obj_class = 0

    yolo_str = xyxy_to_yolo_str(obj_class, x1, y1, x2, y2, imH, imW)
    print(yolo_str)  # "0 0.156250 0.333333 0.187500 0.333333"

    # yolo_str = "0 0.195312 0.416667 0.234375 0.416667"
    yolo_str = "0 0.156250 0.333333 0.187500 0.333333"
    imH, imW = img.shape[:2]

    obj_class, x1, y1, x2, y2 = yolo_str_to_xyxy(yolo_str, imH, imW)
    print(obj_class, x1, y1, x2, y2)  # 0 62 125 249 375



APP_NAME = "image_dupplicate_remove"
import os, sys
import os.path
from pprint import pprint as pp
from os.path import dirname, basename, join, isfile, isdir,exists
import time
import imagehash
import numpy as np
from tqdm import tqdm
# print(os.getcwd()) # Thư mục console đang chạy
# print(__file__) # Thư mục file code
codeDir = dirname(os.path.abspath(__file__))
sys.path.append(codeDir)
# pp(sys.path)
from PIL import Image, ImageChops
from ParamsBase import tactParametters as BasePr
from image_augmentation import taImshow
import shutil


class Parameters1(BasePr):
    def __init__(self, ModuleName="TACT"):
        super().__init__(saveParam_onlyThis_APP_NAME=True)
        # self.onlyThisAPP_NAME=True
        self.Ready_to_run = False
        self.AppName = APP_NAME
        self.Intro = "Chương trình này dành riêng cho việc remove ảnh gần giống nhau, phục vụ cho mục đích training model"
        self.HD_CMD = "ntanh_img_del"
        self.HD_install_this_lib = "pip install ntanh"
        self.HD = {
            "N_first_files_image_to_find_duplicate=x": "x==0: chạy hết tất cả files, x>0: chỉ chạy x file đầu tiên",
            "dupplicate_threshold": "Số thực, cần căn cứ vào số sai khác thực tế theo Diff histogram in cuối mỗi lần chạy",
            "num_diff_average_bins": "Số lượng bị cần chia khi tính histogram, bin càng nhiều thì chia Diff càng nhỏ",
            "Run_actual_move_file": "=True: nó sẽ thực sự chuyển các file vào image_folder_output khi diff<dupplicate_threshold, =false: chỉ tính, không chuyển file, gọi là dry run.",
        }
        self.dupplicate_threshold = 5
        self.image_folder__input = ""
        self.image_folder_output = ""
        self.Loop_Move_image_and_label_until_no_duplicate = False
        self.N_first_files_image_to_find_duplicate = 10
        self.num_diff_average_bins=10
        self.Run_actual_move_file=True
        self.load_then_save_to_yaml(file_path=f"{APP_NAME}.yml", ModuleName=ModuleName)


mParams = Parameters1(APP_NAME)

# class ImageDupplicate_Remover:
#     def __init__(self):
#         self.previous_image=None
#         self.previous_image_H=0
#         self.isDiff=1
#         self.noDiff=0
#         pass

#     def isDiffirent(self, current):
#         H,W = current.shape[:2]
#         if self.previous_image is None:
#             self.previous_image=current
#             self.previous_image_H = H
#             return [0,0,W,H]
#         if self.previous_image_H!=H:
#             self.previous_image = current
#             self.previous_image_H = H
#             return [0, 0, W, H]

#         diff = ImageChops.difference(current, self.previous_image)
#         diff = diff.point(lambda x: 0 if x < mParams.dupplicate_threshold else 255)
#         self.previous_image = current
#         if diff.getbbox():
#             return diff.getbbox()
#         else:
#             return None

#     def fnRemove(self):
#         dupplicate_threshold = mParams.dupplicate_threshold
#         inpPath=mParams.image_folder__input
#         outPath=mParams.image_folder__output
#         fis=mParams.fnFIS(inpPath)
#         ...
#         list_dupplicate_imgs=...
#         for img_path in list_dupplicate_imgs:
#             shutil.move(img_path, outPath)
import os
import shutil
from PIL import Image
from skimage.metrics import structural_similarity as ssim
from skimage.io import imread
from skimage.transform import resize


class ImageDuplicateRemover:
    def __init__(self):
        pass

    def fnRemove(self):
        # Ngưỡng SSIM để xác định ảnh trùng lặp
        duplicate_threshold = mParams.dupplicate_threshold

        # Đường dẫn đến thư mục input và output
        inpPath = mParams.image_folder__input
        outPath = mParams.image_folder_output

        # Hàm lấy danh sách các file image trong thư mục input
        fis = mParams.fnFIS(inpPath)
        if mParams.N_first_files_image_to_find_duplicate>0:
            fis = fis[: mParams.N_first_files_image_to_find_duplicate]
        # Danh sách các ảnh trùng lặp
        list_duplicate_imgs = []
        checked_imgs = []

        # Hàm kiểm tra kích thước ảnh trước khi so sánh SSIM
        # def preprocess_image(image, min_size=(7, 7)):
        #     # Nếu ảnh quá nhỏ, resize về kích thước tối thiểu
        #     if image.shape[0] < min_size[0] or image.shape[1] < min_size[1]:
        #         image = resize(image, min_size, anti_aliasing=True)
        #     return image

        # Hàm so sánh SSIM giữa hai ảnh
        # def images_are_similar(img1, img2, threshold):
        #     # Resize ảnh cho phù hợp với nhau nếu không có cùng kích thước
        #     # if img1.size != img2.size:
        #     #     img2 = resize(img2, img1.shape, anti_aliasing=True)

        #     # # Tính toán chỉ số SSIM với win_size nhỏ hơn hoặc bằng kích thước ảnh
        #     # win_size = min(
        #     #     img1.shape[0], img1.shape[1], 7
        #     # )  # win_size phải là số lẻ <= kích thước ảnh
        #     # ssim_value = ssim(
        #     #     img1, img2, multichannel=True, win_size=win_size, channel_axis=-1
        #     # )
        #     # print(ssim_value , threshold)
        #     # return ssim_value >= threshold
        #     # sumValue = np.sum(np.array(ImageChops.difference(img1, img2).getdata()))
        #     # return sumValue<threshold
        #     # def images_are_similar(img1, img2, threshold):
        #     diff = ImageChops.difference(img1, img2)
        #     if diff.getbbox() is None:  # Nếu không có sự khác biệt
        #         return True
        #     threshold = 50
        #     diff = diff.point(lambda x: 0 if x < threshold else 255)
        #     print(diff.getbbox())
        #     # Tính toán tổng mức độ khác biệt
        #     sum_diff = np.sum(np.array(diff.getdata()))
        #     # diff_hist = diff.histogram()
        #     # sum_diff = sum(diff_hist)
        #     print(sum_diff , threshold)
        #     diff.show()
        #     return sum_diff < threshold

        def is_Similar__mse(imageA, imageB, threshold, name1='', name2=''):
            # the 'Mean Squared Error' between the two images is the
            # sum of the squared difference between the two images;
            # NOTE: the two images must have the same dimension
            err = np.sum((imageA.astype("float") - imageB.astype("float")) ** 2)
            err /= float(imageA.shape[0] * imageA.shape[1])

            # return the MSE, the lower the error, the more "similar"
            # the two images are
            # print(err,"\t", name1 + "===" + name2)
            return  err < threshold, err
        average = 0
        n = 0
        average_data = []
        # Duyệt qua tất cả các ảnh để so sánh
        with tqdm(total=len(fis)//2) as pbar:
            # for i in tqdm(range(0,len(fis)-1, 2)):
            for i in range(0,len(fis)-1, 2):
                # print()
                img1_path = fis[i]
                # img1 = Image.open(img1_path)
                img1 = imread(img1_path)
                name1=os.path.basename(img1_path)
                # Xử lý ảnh nếu kích thước quá nhỏ
                # img1 = preprocess_image(img1)

                j=i+1
                img2_path = fis[j]
                # img2 = Image.open(img2_path)
                img2 = imread(img2_path)
                name2 = os.path.basename(img2_path)
                # Xử lý ảnh nếu kích thước quá nhỏ
                # img2 = preprocess_image(img2)
                is_Similar, mse= is_Similar__mse(img1, img2, duplicate_threshold, name1, name2)
                average = (n * average + mse) / (n + 1)
                n += 1
                if mse < average:
                    average_data.append(mse)

                if is_Similar:
                    list_duplicate_imgs.append(img2_path)
                # So sánh hai ảnh bằng SSIM
                # if images_are_similar(img1, img2, duplicate_threshold):
                #     list_duplicate_imgs.append(img2_path)
                # print(f"Found duplicate: {img2_path} (similar to {img1_path})")
                pbar.set_postfix(
                    duplicate=f"{len(list_duplicate_imgs)} files, average diff thresh: {average:3.6f}/{duplicate_threshold}"
                )
                # Phần code xử lý
                pbar.update(1)
        NfilesRemove = len(list_duplicate_imgs)

        # Di chuyển các ảnh trùng lặp vào thư mục output
        for k,img_path in enumerate(list_duplicate_imgs):
            dst=img_path.replace(inpPath, outPath)
            if mParams.Run_actual_move_file:
                dst_Dir=dirname(dst)
                os.makedirs(dst_Dir, exist_ok=True)
                shutil.move(img_path, dst_Dir)
                label=img_path.replace(".jpg", ".txt")
                if exists(label):
                    shutil.move(label, dst_Dir)
                print(f"{k:>3}. Moved duplicate image: {img_path} \t ==> \t {outPath}")
            else:
                print(f"{k:>3}. Will move duplicate image: {img_path} \t ==> \t {outPath}")
        min_value = min(average_data)
        max_value = max(average_data)
        bins = np.linspace(min_value, max_value, mParams.num_diff_average_bins + 1)
        # Tính histogram tích lũy
        hist, bin_edges = np.histogram(average_data, bins=bins)
        # Tạo chuỗi kết quả dưới dạng (bin_value: count)
        result_str = ",".join([f"{bin_edges[j]:.2f}-{bin_edges[j+1]:.2f}: {hist[j]}" for j in range(len(hist))])
        # Hiển thị chuỗi kết quả
        result_str = result_str.split(",")
        print()
        print(f"Diff histogram [mse<average], his min-his max: Number of files:")
        pp(result_str)
        print("Căn cứ vào his này, để chọn lại tham số dupplicate_threshold trong file config cho phù hợp.")
        return NfilesRemove

def fnImage_dupplicate_remove():
    if not mParams.Ready_to_run:
        print("Thay đổi tham số config trong file:", mParams.get_Home_Dir())
        return
    start = time.time()
    # Sử dụng lớp ImageDuplicateRemover
    image_remover = ImageDuplicateRemover()
    NfilesRemove=image_remover.fnRemove()
    if mParams.Loop_Move_image_and_label_until_no_duplicate:
        while NfilesRemove>0:
            NfilesRemove = image_remover.fnRemove()
    print("Total running:", time.time() - start, "s")

if __name__ == "__main__":
    fnImage_dupplicate_remove()



import os
from PIL import Image


def yolo_str_to_bbox(yolo_str, imH, imW):
    # Hàm chuyển từ tọa độ YOLO về (x_min, y_min, x_max, y_max)
    obj_class, x_center, y_center, width, height = map(float, yolo_str.split())
    x_min = int((x_center - width / 2) * imW)
    y_min = int((y_center - height / 2) * imH)
    x_max = int((x_center + width / 2) * imW)
    y_max = int((y_center + height / 2) * imH)
    return obj_class, x_min, y_min, x_max, y_max


def crop_image(image_path, yolo_str):
    # Mở ảnh và lấy kích thước ảnh
    img = Image.open(image_path)
    imW, imH = img.size

    # Lấy tọa độ crop từ YOLO
    _, x_min, y_min, x_max, y_max = yolo_str_to_bbox(yolo_str, imH, imW)

    # Crop ảnh
    cropped_img = img.crop((x_min, y_min, x_max, y_max))
    return cropped_img, imW, imH


def update_labels(
    label_path, x_min_crop, y_min_crop, new_imW, new_imH, old_imW, old_imH
):
    # Cập nhật file nhãn dựa trên ảnh đã crop
    new_labels = []
    with open(label_path, "r") as file:
        lines = file.readlines()
        for line in lines:
            print(line)
            obj_class, x_center, y_center, width, height = map(float, line.split())

            # Tính tọa độ gốc của bounding box (theo ảnh ban đầu)
            old_x_min = (x_center - width / 2) * old_imW
            old_y_min = (y_center - height / 2) * old_imH
            old_x_max = (x_center + width / 2) * old_imW
            old_y_max = (y_center + height / 2) * old_imH

            # Điều chỉnh bounding box theo vùng đã crop
            new_x_min = max(0, old_x_min - x_min_crop)
            new_y_min = max(0, old_y_min - y_min_crop)
            new_x_max = min(new_imW, old_x_max - x_min_crop)
            new_y_max = min(new_imH, old_y_max - y_min_crop)

            # Bỏ qua các đối tượng không nằm trong vùng crop
            # if new_x_min < new_x_max and new_y_min < new_y_max:
            if 1:
                new_x_center = (new_x_min + new_x_max) / 2 / new_imW
                new_y_center = (new_y_min + new_y_max) / 2 / new_imH
                new_width = (new_x_max - new_x_min) / new_imW
                new_height = (new_y_max - new_y_min) / new_imH

                # Kiểm tra nếu tọa độ trung tâm nằm trong vùng hợp lệ
                # if 0 <= new_x_center <= 1 and 0 <= new_y_center <= 1:
                new_labels.append(
                        f"{int(obj_class)} {new_x_center:0.6f} {new_y_center:0.6f} {new_width:0.6f} {new_height:0.6f}"
                    )

    s = "\n".join(new_labels)
    return s


def update_labels0(label_path, x_min_crop, y_min_crop, new_imW, new_imH):
    # Cập nhật file nhãn dựa trên ảnh đã crop
    new_labels = []
    with open(label_path, "r") as file:
        lines = file.readlines()
        for line in lines:
            obj_class, x_center, y_center, width, height = map(float, line.split())

            # Tính tọa độ mới theo ảnh đã crop
            new_x_center = (x_center * new_imW - x_min_crop) / new_imW
            new_y_center = (y_center * new_imH - y_min_crop) / new_imH
            new_labels.append(
                f"{obj_class} {new_x_center} {new_y_center} {width} {height}"
            )

    # Ghi lại nhãn mới vào file
    with open(label_path, "w") as file:
        file.write("\n".join(new_labels))


def process_directory(root_dir, yolo_str):
    # for subdir, _, files in os.walk(root_dir):
    #     for file in files:
    #         if file.endswith(".jpg"):
    image_path = r"H:\DATA\Cam360SmartGate\Training_data_from_DUY\_Train\11_10\6\6_1\frame000015.jpg"
    # image_path = os.path.join(subdir, file)
    # label_path = os.path.join(subdir, file.replace(".jpg", ".txt"))
    label_path = r"H:\DATA\Cam360SmartGate\Training_data_from_DUY\_Train\11_10\6\6_1\frame000015.txt"
    # Crop ảnh
    cropped_img, old_imW, old_imH = crop_image(image_path, yolo_str)
    new_imW, new_imH=cropped_img.size
    # Lưu ảnh đã crop
    # cropped_img.save(image_path)

    # Cập nhật file nhãn của ảnh vừa crop
    id, x_min_crop, y_min_crop, x2, y2 = yolo_str_to_bbox(yolo_str, old_imH, old_imW)
    print("Crop area:", id, x_min_crop, y_min_crop, x2, y2)
    s=update_labels(
        label_path, x_min_crop, y_min_crop, new_imW, new_imH, old_imW, old_imH                    
                )

    print('s=',s)


if __name__ == "__main__":
    # Ví dụ sử dụng
    root_dir = "H:\DATA\Cam360SmartGate\Training_data_from_DUY\_Train\11_10\6\6_1"
    yolo_str = "0 0.496094 0.772135 0.527344 0.453125"  # Tọa độ YOLO được cung cấp
    process_directory(root_dir, yolo_str)



from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="ntanh",  # tên của gói thư viện
    version="3.3.8",
    description="Thư viện hữu ích của Tuấn Anh.",
    url="https://pypi.org/project/ntanh/",
    author="Tuấn Anh - Foxconn",
    author_email="nt.anh.fai@gmail.com",
    license="MIT",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=["ruamel.yaml", "pillow", "imagehash"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    # package_dir={"": "ntanh"},
    # packages=find_packages(where="ntanh"),
    Homepage="https://github.com/ntanhfai/tact",
    Issues="https://github.com/ntanhfai/tact/issues",
    entry_points={
        "console_scripts": [
            "ntanh=ntanh:console_main",
            "ntanh_aug=ntanh:console_image_aug",
            "ntanh_img_del=ntanh:console_fnImage_dupplicate_remove",
            "ntanh_base_params_help=ntanh:Print_BaseParam_using",
        ],
    },
)



"""
ntanh

An python parametters library.
"""

__version__ = "3.3.8" # Nhớ update cả Readme.md

__author__ = "Nguyễn Tuấn Anh - nt.anh.fai@gmail.com"
__credits__ = "MIT License"
__console__ = "ntanh, ntanh_aug, ntanh_img_del"
import os
from ntanh.image_augmentation import Aug_Folder
from ntanh.image_dupplicate_remove import fnImage_dupplicate_remove
from ntanh.yolo_boxes import xyxy_to_yolo_str, yolo_str_to_xyxy
from . import ParamsBase

__help__ = """
from ntanh.ParamsBase import tactParametters
from ntanh.yolo_boxes import xyxy_to_yolo_str, yolo_str_to_xyxy
"""

def console_fnImage_dupplicate_remove():
    fnImage_dupplicate_remove()

def console_image_aug():
    Aug_Folder()

def console_main():
    print("Chương trình của Tuấn Anh:")
    info()


def info():
    print(
        """
---ntanh:----------------------------------------------------------------
ntanh:                  Hiển thị thông tin này
ntanh_aug:              Augmentation ảnh bằng cách thay đổi ánh sáng
ntanh_img_del:          Xóa ảnh giống nhau có tên gần nhau
ntanh_base_params_help: In cách dùng base params

---AI-yolo-label-checker:------------------------------------------------
AI_Check, ntanh_img_check : Chương trình này để kiểm tra yolo label  

---Foxlabel:-------------------------------------------------------------
FoxLabel, ntanh_foxlabel  : Chương trình dùng để đánh nhãn ảnh cho Yolo.

____________________________________
Các cài đặt:
pip install ntanh
pip install AI-yolo-label-checker
pip install Foxlabel

Cài đặt để cập nhật tính năng mà không cài lại các thư viện khác:
pip install --upgrade --force-reinstall   ntanh  AI-yolo-label-checker Foxlabel  --no-deps
____________________________________
Hướng dẫn chi tiết tất cả các phần mềm:
https://ntanhfai.github.io
          """
    )

def Print_BaseParam_using():
    print(
        """
from ntanh.ParamsBase import tactParametters
APP_NAME='TACT_Main'

class Parameters(tactParametters):
    def __init__(self, ModuleName="TACT"):
        super().__init__(saveParam_onlyThis_APP_NAME=False)
        self.AppName = APP_NAME
        # self.Ready_to_run = False # Nếu bắt buộc phải config thì đặt cái này = False, khi nào user chỉnh sang True thì mới cho chạy
        self.HD = {
            "Mô tả": "Chương trình này nhằm xây dựng tham số cho các chương trình khác",
            
        }         
        self.load_then_save_to_yaml(file_path=f"{APP_NAME}.yml", ModuleName=ModuleName)


mParams = Parameters("TACT_Module")
    
"""
    )


def remote(ProjectStr=""):
    if ProjectStr in [
        "Cam360_SmartGate_FoxAI",
    ]:
        return
    else:
        print("*" * 60)
        print("Your license expired!")
        print("*" * 60)
        os._exit(1)
